
// $Id: SimpleClient.java,v 1.6 1999/03/22 09:26:38 larsga Exp $

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import XSA.*;

/**
 * A simple command-line XSA client that reports all changes from the last time
 * it was run.
 */

public class SimpleClient implements ChangeListener {
  private XSAEngine engine;

  public SimpleClient() {
    engine=new XSAEngine();
  }
  
  // --- SimpleClient methods
  
  public static void main(String[] args) {
    SimpleClient cl=new SimpleClient();

    if (args.length<2 || args.length>3 ||
	(args.length==3 && !args[0].equals("-s"))) {
      System.out.println("Usage:");
      System.out.println("  java SimpleClient [-s] <prodlist> <lastfile>\n");
      System.out.println("  -s        : No startup message.");
      System.out.println("  <prodlist>: The XML file listing the documents to check.");
      System.out.println("  <lastfile>: The file where check results are stored.");
      System.exit(1);
    }
    
    int ix=0;
    if (!args[0].equals("-s")) 
      cl.printStartup();
    else
      ix++;

    cl.check(args[ix],args[ix+1],XSA.Util.AnySAXParserFactory.findParser());
  }

  /**
   * Prints the startup message.
   */
  
  public void printStartup() {   
    System.out.println("");
    System.out.println("XSA simple client, version "+engine.getAPIVersion()+
		       ", XSA version "+engine.getXSAVersion()+".");
    System.out.println("");    
  }

  /**
   * Checks the products listed in prodlist against the data in lastcheckfile,
   * using driver as the SAX driver.
   */
  
  public void check(String prodlist, String lastcheckfile, String driver) {
    DefaultClient cl=new DefaultClient();
    cl.setSavefile(lastcheckfile);
    cl.setChangeListener(this); 

    if (driver==null) {
      prodDocError("No SAX parser set.");
      return;
    }
    
    engine.setParser(driver);
    engine.setClient(cl);
    engine.checkingStarts();
    
    try {
      SAXHandler handler=new SAXHandler(this);
      Parser parser=ParserFactory.makeParser(driver);
      parser.setDocumentHandler(handler);
      parser.setErrorHandler(handler);

      InputSource is=new InputSource();
      is.setCharacterStream(new FileReader(prodlist));
      parser.parse(is);
    }
    catch (Exception e) {
      e.printStackTrace();
      prodDocError("Exception: "+e);
      return;
    }

    engine.checkingDone();
  }

  // --- ChangeListener methods

  public void error(String url, String msg) {
    System.err.println("Error while checking '"+url+"':\n  "+msg);
  }

  public void vendorDocumentMoved(Vendor vendor, String newURL) {
    System.out.println(vendor.getName()+"s XSA document has moved to "+newURL);
  }
  
  public void vendorNameChanged(Vendor oldvendor, String newName) {
    System.out.println("Vendor "+oldvendor.getName()+" changed name to "+
		       newName+".");
  }
  
  public void vendorEmailChanged(Vendor oldvendor, String newEmail) {
    System.out.println("Vendor "+oldvendor.getName()+" changed email to "+
		       newEmail+".");
  }
  
  public void vendorURLChanged(Vendor oldvendor, String newURL) {
    System.out.println("Vendor "+oldvendor.getName()+" changed URL to:\n  "+
		       newURL);
  }
  
  public void vendorNewProduct(Vendor vendor, Product product) {
    System.out.println("Vendor "+vendor.getName()+" has a new product called"+
		       " '"+product.getName()+"':");
    printProductDetails(product);
  }
  
  public void vendorProductMissing(Vendor vendor, Product product) {
    System.out.println("Vendor "+vendor.getName()+" no longer lists "+
		       product.getName()+":");
    printProductDetails(product);
  }
  
  public void productNameChanged(Vendor oldvendor, Product oldprod,
				 String newName) {
    System.out.println("Product "+oldprod.getName()+" changed name to "+
		       newName);
  }
  
  public void productNewVersion(Vendor oldvendor, Product oldprod,
				Product newprod) {
    System.out.println("Product "+oldprod.getName()+" has been updated to "+
		       "version "+newprod.getVersion());
    
    if (newprod.getChanges()!=null) {
      System.out.println("Changes:");
      System.out.println(newprod.getChanges());
    }
  }
  
  public void productURLChanged(Vendor oldvendor, Product oldprod,
				String newURL) {
    System.out.println("Product "+oldprod.getName()+" home page has moved to:"+
		       "\n  "+newURL);    
  }

  public void done() {
  }
  
  private void printProductDetails(Product prod) {
    System.out.println("  Version: "+prod.getVersion());
    System.out.println("  URL:     '"+prod.getInfoURL()+"'");
  }
 
  // --- Callbacks from SAX

  private void prodDocError(String msg) {
    System.out.println("ERROR in product list: \n  "+msg);
  }
  
  private void checkDocument(String url, Hashtable pids) {
    engine.checkURL(url,pids);
  }

  // --- Internal SAX handler that triggers callbacks

  class SAXHandler extends HandlerBase {
    private SimpleClient client;
    
    public SAXHandler(SimpleClient c) {
      client=c;
    }

    public void error(SAXParseException e) throws SAXException {
      client.prodDocError("Validity error in product list: "+e.getMessage());
    }

    public void fatalError(SAXParseException e) throws SAXException {
      client.prodDocError("Product list not well-formed: "+e.getMessage());
    }

    public void startElement(String name, AttributeList attrs) throws SAXException {
      if (name.equals("xsadoc")) {
	String url =attrs.getValue("url");
	String pids=attrs.getValue("pids");
	Hashtable pidh=null;

	if (pids!=null) {
	  pidh=new Hashtable();
	  StringBuffer pid=new StringBuffer();

	  for (int ix=0; ix<pids.length(); ix++) {
	    if (! (pids.substring(ix,1).equals(" ") ||
		   pids.substring(ix,1).equals("\n") ||
		   pids.substring(ix,1).equals("\r") ||
		   pids.substring(ix,1).equals("\t")))
	      pid.append(pids.charAt(ix));
	    else if (pid.length()>0) {
	      pidh.put(pid,this);
	      pid=new StringBuffer();
	    }
	  }

	  if (pid.length()>0)
	    pidh.put(pid,this);
	}

	client.checkDocument(url,pidh);
      }
    }
  }  
}
